import React, { Component } from 'react';
import { Animated, PanResponder, Dimensions, Image, View, Text } from 'react-native';

const SCREEN_WIDTH = Dimensions.get('window').width;
const SCREEN_HEIGHT = Dimensions.get('window').height;
const HORIZ_SWIPE_THRESHOLD = 0.25 * SCREEN_WIDTH;
const VERT_SWIPE_THRESHOLD = 0.25 * SCREEN_HEIGHT;
const SWIPE_OUT_DURATION = 205;

function mod(x, n) {
  // handles x < 0 gracefully
  return ((x % n) + n) % n;
}

class Deck extends Component {
  static defaultProps = {
    onSwipeRight: () => {},
    onSwipeLeft: () => {},
    onSwipeUp: () => {},
    onSwipeDown: () => {},
  };

  constructor(props) {
    // console.log('props before super', props);

    super(props);

    this.forceSwipe = this.forceSwipe.bind(this);
    this.onSwipeComplete = this.onSwipeComplete.bind(this);
    this.finishSwipe = this.finishSwipe.bind(this);
    this.createPanResponder = this.createPanResponder.bind(this);
    this.createState = this.createState.bind(this);
    this.prevCardDefaultCoordinates = this.prevCardDefaultCoordinates.bind(
      this
    );

    this.state = {
      ...this.createState(),
      cardIndex: 0,
    };

    // console.log('props at the end of the constructor', props);
  }

  componentDidUpdate = (_, prevState) => {
    if (prevState.cardIndex !== this.state.cardIndex) {
      this.setState(this.createState());
    }
  }

  onSwipeComplete(direction) {
    const {
      onSwipeLeft,
      onSwipeRight,
      onSwipeUp,
      onSwipeDown,
      data,
    } = this.props;

    // console.log(this.props);

    const item = data[this.state.cardIndex];
    if (direction === 'right') {
      onSwipeRight(item);
      this.finishSwipe('prev');
    } else if (direction === 'left') {
      onSwipeLeft(item);
      this.finishSwipe('next');
    } else if (direction === 'up') {
      onSwipeUp(item);
      this.finishSwipe('prev');
    } else {
      onSwipeDown(item);
      this.finishSwipe('next');
    }
  }

  getCurrentCardStyle() {
    const { currentCardPosition } = this.state;
    return {
      ...currentCardPosition.getLayout(),
    };
  }

  getPrevCardStyle() {
    const { prevCardPosition } = this.state;
    return {
      ...prevCardPosition.getLayout(),
    };
  }

  forceSwipe(direction) {
    if (this.props.swipeOrientation === 'horiz') {
      if (direction === 'left') {
        Animated.timing(this.state.currentCardPosition, {
          toValue: this.prevCardDefaultCoordinates(),
          duration: SWIPE_OUT_DURATION,
        }).start(() => this.onSwipeComplete(direction));
      } else if (direction === 'right') {
        Animated.timing(this.state.prevCardPosition, {
          toValue: { x: 0, y: 0 },
          duration: SWIPE_OUT_DURATION,
        }).start(() => this.onSwipeComplete(direction));
      }
    } else if (this.props.swipeOrientation === 'vert') {
      if (direction === 'down') {
        Animated.timing(this.state.currentCardPosition, {
          toValue: this.prevCardDefaultCoordinates(),
          duration: SWIPE_OUT_DURATION,
        }).start(() => this.onSwipeComplete(direction));
      } else if (direction === 'up') {
        Animated.timing(this.state.prevCardPosition, {
          toValue: { x: 0, y: 0 },
          duration: SWIPE_OUT_DURATION,
        }).start(() => this.onSwipeComplete(direction));
      }
    }
  }

  finishSwipe(endingCard) {
    const len = this.props.data.length;
    const cardDelta = endingCard === 'next' ? +1 : -1; // otherwise previous
    this.setState({
      cardIndex: mod(this.state.cardIndex + cardDelta, len),
    });
  }

  resetPosition() {
    Animated.spring(this.state.currentCardPosition, {
      toValue: { x: 0, y: 0 },
    }).start();
    Animated.spring(this.state.prevCardPosition, {
      toValue: this.prevCardDefaultCoordinates(),
    }).start();
  }

  createPanResponder(currentCardPosition, prevCardPosition) {
    const horiz = this.props.swipeOrientation === 'horiz';
    const vert = this.props.swipeOrientation === 'vert';

    return PanResponder.create({
      onStartShouldSetPanResponder: () => true,
      onPanResponderMove: (event, gesture) => {
        const left = gesture.dx < 0;
        const right = gesture.dx > 0;
        const up = gesture.dy < 0;
        const down = gesture.dy > 0;
        if (horiz) {
          if (left) {
            currentCardPosition.setValue({ x: gesture.dx, y: 0 });
          } else if (right) {
            prevCardPosition.setValue({ x: gesture.dx - SCREEN_WIDTH, y: 0 });
          }
        } else if (vert) {
          if (down) {
            currentCardPosition.setValue({ x: 0, y: gesture.dy });
          } else if (up) {
            prevCardPosition.setValue({ x: 0, y: gesture.dy + SCREEN_HEIGHT });
          }
        }
      },
      onPanResponderRelease: (event, gesture) => {
        if (horiz) {
          if (gesture.dx > HORIZ_SWIPE_THRESHOLD) {
            this.forceSwipe('right');
          } else if (gesture.dx < -HORIZ_SWIPE_THRESHOLD) {
            this.forceSwipe('left');
          } else {
            this.resetPosition();
          }
        } else if (vert) {
          if (gesture.dy > VERT_SWIPE_THRESHOLD) {
            this.forceSwipe('down');
          } else if (gesture.dy < -VERT_SWIPE_THRESHOLD) {
            this.forceSwipe('up');
          } else {
            this.resetPosition();
          }
        } else {
          // bad props => swipeOrientation needs to be 'horiz' or 'vert'
        }
      },
    });
  }

  createState() {
    const currentCardPosition = new Animated.ValueXY();
    const prevCardPosition = new Animated.ValueXY(
      this.prevCardDefaultCoordinates()
    );
    const panResponder = this.createPanResponder(
      currentCardPosition,
      prevCardPosition
    );
    return {
      currentCardPosition,
      prevCardPosition,
      panResponder,
    };
  }

  prevCardDefaultCoordinates() {
    if (this.props.swipeOrientation === 'horiz') {
      return {
        x: -SCREEN_WIDTH,
        y: 0,
        dy: 0,
      };
    } else if (this.props.swipeOrientation === 'vert') {
      return {
        x: 0,
        y: SCREEN_HEIGHT,
        dy: 0,
      };
    }
    return null;
  }

  renderCard(item) {
    return (
      <View style={styles.deckCard}>
        <Image
          source={{ uri: item.file }}
          style={{ width: SCREEN_WIDTH * 0.90, height: SCREEN_HEIGHT * 0.90 }}
        />
      </View>
    );
  }

  renderCards() {
    const numCards = this.props.data.length;

    const currentCardIndex = this.state.cardIndex;
    const currentCardItem = this.props.data[currentCardIndex];

    if (numCards === 1) {
      return [(
        <View style={[this.getCurrentCardStyle(), styles.cardStyle]}>
          {this.renderCard(currentCardItem)}
        </View>
      )];
    }

    const currentCardComponent = (
      <Animated.View
        key={1}
        style={[this.getCurrentCardStyle(), styles.cardStyle]}
        {...this.state.panResponder.panHandlers}
      >
        {this.renderCard(currentCardItem)}
      </Animated.View>
    );

    const prevCardIndex = mod(currentCardIndex - 1, numCards);
    const nextCardIndex = mod(currentCardIndex + 1, numCards);
    const prevCardItem = this.props.data[prevCardIndex];
    const nextCardItem = this.props.data[nextCardIndex];

    const prevCardComponent = (
      <Animated.View
        key={0}
        style={[this.getPrevCardStyle(), styles.cardStyle]}
      >
        {this.renderCard(prevCardItem)}
      </Animated.View>
    );
    const nextCardComponent = (
      <Animated.View key={2} style={styles.cardStyle}>
        {this.renderCard(nextCardItem)}
      </Animated.View>
    );
    return [nextCardComponent, currentCardComponent, prevCardComponent];
  }

  render() {
    if (this.props.data.length === 0) {
      return null;
    }

    return (
      <Animated.View style={{ flex: 1 }}>
        {this.renderCards()}
        <View
          style={{
            position: 'absolute',
            bottom: -40,
            left: SCREEN_WIDTH - 48,
            zIndex: 20,
          }}
        >
          <Text
            style={{
              color: '#9A9A9A',
              fontSize: 12,
              fontFamily: 'Gustan-Book',
            }}
          >
            {(this.state.cardIndex + 1).toString()}/
            {this.props.data.length.toString()}
          </Text>
        </View>
      </Animated.View>

    );
  }
}

const styles = {
  cardStyle: {
    position: 'absolute',
    width: SCREEN_WIDTH,
  },
  deckCard: {
    flex: 1,
    alignItems: 'center',
    justifyContent: 'center',
    bottom: 50,
  },
};

export default Deck;
